import rclpy, re
from rclpy.node import Node
from rclpy.qos import QoSProfile

from sensor_msgs.msg import JointState

from math import pi

from std_msgs.msg import Float32

from serial import Serial

from enum import Enum

class Servos(Enum):
    J1 = 'A'
    J2 = 'B'
    J3 = 'C'

class CobotMove(Node):
    def __init__(self):
        super().__init__('cobot_move')
        self._logger = self.get_logger()
        self._logger.info('COBOT init')

        self._serial = Serial('/dev/ttyACM0', 115200)

        self._cur = [0.0] * 6
        # self._buf = [90, 90, 90] # 3 servos
        # self._prev_buf = [90, 90, 90]

        base_topic = 'move/'

        self.create_subscription(Float32, base_topic+'j1', self._move_j1_cb, rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'j2', self._move_j2_cb, rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'j3', self._move_j3_cb, rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'j4', self._move_j4_cb, rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'j5', self._move_j5_cb, rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'j6', self._move_j6_cb, rclpy.qos.qos_profile_sensor_data)

        publish_base_topic = 'update/'
        self.s_j1 = self.create_publisher(Float32, publish_base_topic+'j1', rclpy.qos.qos_profile_sensor_data)
        self.s_j2 = self.create_publisher(Float32, publish_base_topic+'j2', rclpy.qos.qos_profile_sensor_data)
        self.s_j3 = self.create_publisher(Float32, publish_base_topic+'j3', rclpy.qos.qos_profile_sensor_data)
        self.s_j4 = self.create_publisher(Float32, publish_base_topic+'j4', rclpy.qos.qos_profile_sensor_data)
        self.s_j5 = self.create_publisher(Float32, publish_base_topic+'j5', rclpy.qos.qos_profile_sensor_data)
        self.s_j6 = self.create_publisher(Float32, publish_base_topic+'j6', rclpy.qos.qos_profile_sensor_data)

        qos = QoSProfile(depth=10)
        self._joint_pub = self.create_publisher(JointState, 'joint_states', qos)
        self._joint_state = JointState()

        self._serial.write(b'STA0.0.1\n')
        print(self._serial.readline())

    def _move_j1_cb(self, msg):
        self._cur[0] = int(msg.data / pi / 2. * 8000)
        self._move_servo()

    def _move_j2_cb(self, msg):
        self._cur[1] = int(msg.data / pi / 2. * 10000)
        self._move_servo()

    def _move_j3_cb(self, msg):
        self._cur[2] = int(msg.data / pi / 2. * 10000)
        self._move_servo()

    def _move_j4_cb(self, msg):
        self._cur[3] = int(msg.data / pi / 2. * 2000)
        self._move_servo()

    def _move_j5_cb(self, msg):
        self._cur[4] = int(msg.data / pi / 2. * 2000)
        self._move_servo()

    def _move_j6_cb(self, msg):
        self._cur[5] = int(msg.data / pi / 2. * 2000)
        self._move_servo()

    def _move_servo(self):
        self._serial.write(f'MTA{self._cur[0]}B{self._cur[1]}C{self._cur[2]}D{self._cur[3]}E{self._cur[4]}F{self._cur[5]}\n'.encode('UTF-8'))


        line = self._serial.readline().decode('UTF-8')

        blocks = re.findall(r'[ABCDEF]-*\d+', line)
        a = float(blocks[0][1:]) / 8000. * pi * 2.
        b = float(blocks[1][1:]) / 10000 * pi * 2.
        c = float(blocks[2][1:]) / 10000. * pi * 2.
        d = float(blocks[3][1:]) / 2000. * pi * 2.
        e = float(blocks[4][1:]) / 2000. * pi * 2.
        f = float(blocks[5][1:]) / 2000. * pi * 2.

        self._joint_state.header.stamp = self.get_clock().now().to_msg()
        self._joint_state.name = ['joint_1', 'joint_2', 'joint_3', 'joint_4', 'joint_5', 'joint_6']
        self._joint_state.position = [a, b, c, d, e, f]

        self._joint_pub.publish(self._joint_state)

        self.s_j1.publish(Float32(data=a))
        self.s_j2.publish(Float32(data=b))
        self.s_j3.publish(Float32(data=c))
        self.s_j4.publish(Float32(data=d))
        self.s_j5.publish(Float32(data=e))
        self.s_j6.publish(Float32(data=f))

    #    self._logger.info(f'published {a} {b} {c} {d} {e} {f}')
    # def _push ?

def main(args=None):
    rclpy.init(args=args)

    cobot_move = CobotMove()
    rclpy.spin(cobot_move)

    cobot_move.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()

